/*
dttools is Copyright (C) 2002 Douglas Thain
This software is released under the GNU General Public License.
See the file COPYING for details.
*/


#include "discovery.h"

#include "datagram.h"
#include "debug.h"
#include "hash_table.h"

#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

int discovery_advertise( int dport, const char *service )
{
	struct datagram *d;
	char data[DATAGRAM_PAYLOAD_MAX];
	char addr[DATAGRAM_ADDRESS_MAX];
	char rservice[DATAGRAM_PAYLOAD_MAX];
	int rport;
	int timeout = 60;
	int result;
	int fields;

	debug("discovery: advertising %s on port %d",service,dport);

	d = datagram_create( dport );
	if(!d) return 0;

	while(1) {
		result = datagram_recv( d, data, sizeof(data), addr, &rport, timeout );
		if(result>=0) {
			data[result] = 0;
			debug("discovery: %s <- %s:%d",data,addr,rport); 
			fields = sscanf(data,"want %s",rservice);
			if( fields==1 && !strcmp(rservice,service)) {
				sprintf(data,"have %s",service);
				debug("discovery: %s -> %s:%d",data,addr,rport);
				datagram_send( d, data, strlen(data), addr, rport );
			}
		}
	}
}

int discovery_run( int port, const char *service, char **addrs, int max, int timeout )
{
	struct discovery_client *d;
	time_t stoptime;
	int found=0;
	int i;

	d = discovery_client_create(port,service);
	if(!d) return 0;

	stoptime = time(0)+timeout;

	do {
		if(!discovery_client_send(d)) break;
		if(discovery_client_recv(d,addrs[found],1)) {
			for(i=0;i<found;i++) {
				if(!strcmp(addrs[i],addrs[found])) {
					/* already got that one */
					break;
				}
			}
			if(i==found) found++;
		}
	} while( (found<max) && (time(0)<stoptime) );

	discovery_client_delete(d);

	return found;
}

struct discovery_client {
	struct hash_table *table;
	struct datagram *dgram;
	char *service;
	int port;
};

struct discovery_client * discovery_client_create( int port, const char *service )
{
	struct discovery_client *d;

	d = malloc(sizeof(*d));
	if(!d) return 0;

	memset(d,0,sizeof(*d));

	d->table = hash_table_create(127,hash_string);
	if(!d->table) goto failure;

	d->dgram = datagram_create(DATAGRAM_PORT_ANY);
	if(!d->dgram) goto failure;

	d->service = strdup(service);
	if(!d->service) goto failure;

	d->port = port;

	return d;

	failure:
	discovery_client_delete(d);
	return 0;
}

int discovery_client_send( struct discovery_client *d )
{
	char data[DATAGRAM_PAYLOAD_MAX];
	sprintf(data,"want %s",d->service);
	debug("discovery: %s -> %s:%d",data,DATAGRAM_ADDRESS_BROADCAST,d->port);
	return datagram_send( d->dgram, data, strlen(data), DATAGRAM_ADDRESS_BROADCAST, d->port );
}

int discovery_client_recv( struct discovery_client *d, char *addr, int timeout )
{
	char data[DATAGRAM_PAYLOAD_MAX];
	time_t stoptime = time(0)+timeout;
	int result;
	int port;

	do{
		result = datagram_recv(d->dgram,data,sizeof(data),addr,&port,timeout);
		if(result>0) {
			debug("discovery: %s <- %s:%d",data,addr,port);
			if(
			   !strncmp(data,"have ",5) &&
			   !strcmp(&data[5],d->service) &&
			   !hash_table_lookup(d->table,addr) &&
			   hash_table_insert(d->table,addr,addr,0)) {
				return 1;
			}
		}
	} while( time(0) < stoptime );

	return 0;
}

void discovery_client_delete( struct discovery_client *d )
{
	if(d) {
		if(d->table) hash_table_delete(d->table);
		if(d->dgram) datagram_delete(d->dgram);
		if(d->service) free(d->service);
		free(d);
	}
}


