/*
dttools is Copyright (C) 2002 Douglas Thain
This software is released under the GNU General Public License.
See the file COPYING for details.
*/


#include "itable.h"

#include <stdlib.h>
#include <string.h>

struct entry {
	int key;
	void *value;
	struct entry *next;
};

struct itable {
	int bucket_count;
	struct entry **buckets;
};

struct itable * itable_create( int bucket_count )
{
	struct itable *h;
	int i;

	h = (struct itable*) malloc(sizeof(struct itable));
	if(!h) return 0;

	h->bucket_count = bucket_count;
	h->buckets = (struct entry**) malloc( sizeof(struct entry*)*bucket_count );
	if(!h->buckets) {
		free(h);
		return 0;
	}

	for( i=0; i<bucket_count; i++ ) {
		h->buckets[i] = 0;
	}

	return h;
}

void itable_delete( struct itable *h )
{
	struct entry *e, *f;
	int i;

	for( i=0; i<h->bucket_count; i++ ) {
		e = h->buckets[i];
		while(e) {
			f=e->next;
			free(e);
			e=f;
		}
	}

	free(h->buckets);
	free(h);
}


void * itable_lookup( struct itable *h, int key )
{
	struct entry *e;
	int index;
	
	index = key % h->bucket_count;
	e = h->buckets[index];

	while(e) {
		if(key==e->key) {
			return e->value;
		}
		e=e->next;
	}

	return 0;
}

int itable_insert( struct itable *h, int key, const void *value, void **old )
{
	struct entry *e;
	int index;

	if(old) {
		*old = 0;
	}

	index = key % h->bucket_count;
	e = h->buckets[index];

	while(e) {
		if(key==e->key) {
			if(old) {
				*old = e->value;
			}
			e->value = (void*) value;
			return 1;
		}
		e=e->next;
	}

	e = (struct entry*) malloc(sizeof(struct entry));
	if(!e) return 0;

	e->key = key;
	e->value = (void*) value;
	e->next = h->buckets[index];
	h->buckets[index] = e;

	return 1;
}

void * itable_remove( struct itable *h, int key )
{
	struct entry *e,*f;
	void *value;
	int index;

	index = key % h->bucket_count;
	e = h->buckets[index];
	f = 0;

	while(e) {
		if(key==e->key) {
			if(f) {
				f->next = e->next;
			} else {
				h->buckets[index] = e->next;
			}
			value = e->value;
			free(e);
			return value;
		}
		f = e;
		e = e->next;
	}

	return 0;
}

int itable_iterate( struct itable *h, itable_op_t iterator, void *arg )
{
	int i;
	struct entry *e, *f;

	for( i=0; i<h->bucket_count; i++ ) {
		e = h->buckets[i];
		while(e) {
			f = e->next;
			if(!iterator(h,e->key,e->value,arg)) {
				return 0;
			}
			e = f;
		}
	}

	return 1;
}
