#ifndef PAGE_TABLE_H
#define PAGE_TABLE_H

/* There exists a struct page_table, but it is opaque to the user */
struct page_table;

/* This type defines a handler function to be called during a page fault */
typedef void (*page_table_handler_t) ( struct page_table *pt, int page, void *arg );

/* This type defines a handler to be called by page_table_iterate */
typedef int (*page_table_iterator_t) ( struct page_table *pt, void *arg );

/* Create a page_table to manage the data area given by "base" and "pages".  The memory will have initial permissions given by "access", and "handler" will be called during a page fault on this table. */

struct page_table * page_table_create( char *base, int pages, int access, page_table_handler_t handler, void *handler_arg );

/* Delete this page_table.  Deleting a page_table will set the memory permissions back to "ALL" */
void page_table_delete( struct page_table *pt );

/* Get the base address of the memory managed by this page_table */
char *page_table_base( struct page_table *pt );

/* Or, get the page table that has this base address */
struct page_table * page_table_from_base( char *base );

/* Get the number of pages in this table */
int page_table_pages( struct page_table *pt );

/* Get a pointer to the data in page "page" */
char *page_table_data( struct page_table *pt, int page );

/* Set the access permissions for a set of pages in this table */
int page_table_access_set( struct page_table *pt, int page, int npages, int access );

/* Get the access permission for a page in this table */
int page_table_access_get( struct page_table *pt, int page );

/* Get the default page size */
int page_table_pagesize();

/* Perform an action on all existing page tables */
int page_table_iterate( page_table_iterator_t iter, void *arg );

#define PAGE_TABLE_ACCESS_READ 1
#define PAGE_TABLE_ACCESS_WRITE 2
#define PAGE_TABLE_ACCESS_EXEC 4

#define PAGE_TABLE_ACCESS_NONE 0
#define PAGE_TABLE_ACCESS_ALL (PAGE_TABLE_ACCESS_READ|PAGE_TABLE_ACCESS_WRITE|PAGE_TABLE_ACCESS_EXEC)

#endif
